@@ -721,18 +721,21 @@ namespace std {
     inline _Tp
     _Expr<_Clos,_Tp>::sum () const
     {
-        _Tp __s(_Tp());
-        size_t __n (_M_closure.size ());
-        for (size_t __i=0; __i<__n; ++__i) __s += _M_closure[__i];
-        return __s;
+        size_t __n = _M_closure.size();
+        if (__n == 0) return _Tp();
+        else {
+            _Tp __s = _M_closure[--__n];
+            while (__n != 0) __s += _M_closure[--__n];
+            return __s;
+        }
     }
     
     template<class _Dom, typename _Tp>
     inline _Tp
     min (const _Expr<_Dom,_Tp>& __e)
     {
-        size_t __s (__e.size ());
-        _Tp  __m (__e[0]);
+        size_t __s = __e.size ();
+        _Tp  __m = __e[0];
         for (size_t __i=1; __i<__s; ++__i)
             if (__m > __e[__i]) __m = __e[__i];
         return __m;
@@ -742,8 +745,8 @@ namespace std {
     inline _Tp
     max (const _Expr<_Dom,_Tp>& __e)
     {
-        size_t __s (__e.size());
-        _Tp __m (__e[0]);
+        size_t __s = __e.size();
+        _Tp __m = __e[0];
         for (size_t __i=1; __i<__s; ++__i)
             if (__m < __e[__i]) __m = __e[__i];
         return __m;
@@ -922,7 +925,7 @@ _Name(const _Expr<_Dom,typename _Dom::value_type>& __e)                 \
 {                                                                       \
     typedef typename _Dom::value_type _Tp;                              \
     typedef _UnFunClos<_Expr,_Dom> _Closure;                            \
-    return _Expr<_Closure,_Tp> (_Closure (__e, (_Tp(*)(_Tp))(&_Name))); \
+    return _Expr<_Closure,_Tp>(_Closure(__e(), (_Tp(*)(_Tp))(&_Name))); \
 }                                                                       \
                                                                         \
 template<typename _Tp>                                                  \
@@ -941,7 +944,8 @@ _Name(const valarray<_Tp>& __v)                                         \
     _DEFINE_EXPR_UNARY_FUNCTION(sin)
     _DEFINE_EXPR_UNARY_FUNCTION(asin)
     _DEFINE_EXPR_UNARY_FUNCTION(sinh)    
-    _DEFINE_EXPR_UNARY_FUNCTION(tan)    
+    _DEFINE_EXPR_UNARY_FUNCTION(tan)
+    _DEFINE_EXPR_UNARY_FUNCTION(tanh)
     _DEFINE_EXPR_UNARY_FUNCTION(atan)
     _DEFINE_EXPR_UNARY_FUNCTION(exp)    
     _DEFINE_EXPR_UNARY_FUNCTION(log)
